package com.example.flyme.utils.adapter;

import java.util.ArrayList;

import com.example.flyme.utils.R;
import com.meizu.flyme.utils.DateTimeUtils;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.TextView;

public class DateTimeExpandableLvAdapter extends BaseExpandableListAdapter {
	private ArrayList<String> mParentStrList;
	private ArrayList<ArrayList<String>> mChildStrList;
	private Context mExpandableCt;
	private LayoutInflater mExpandableInflater;

	public DateTimeExpandableLvAdapter(Context context) {
		mExpandableCt = context;
		mExpandableInflater = LayoutInflater.from(mExpandableCt);
		initParentData();
	}

	@Override
	public Object getChild(int arg0, int arg1) {
		return mChildStrList.get(arg0).get(arg1);
	}

	@Override
	public long getChildId(int arg0, int arg1) {
		return 0;
	}

	@Override
	public View getChildView(int arg0, int arg1, boolean arg2, View arg3,
			ViewGroup arg4) {
		ViewHolder viewHolder = null;

		if (arg3 == null) {
			viewHolder = new ViewHolder();

			arg3 = mExpandableInflater.inflate(R.layout.datetime_type_show,
					null);
			viewHolder.text1 = (TextView) arg3.findViewById(R.id.text1);

			arg3.setTag(viewHolder);
		} else {
			viewHolder = (ViewHolder) arg3.getTag();
		}

		viewHolder.text1.setText(mChildStrList.get(arg0).get(arg1));
		return arg3;
	}

	@Override
	public int getChildrenCount(int arg0) {
		return mChildStrList.get(arg0).size();
	}

	@Override
	public Object getGroup(int arg0) {
		return mParentStrList.get(arg0);
	}

	@Override
	public int getGroupCount() {
		return mParentStrList.size();
	}

	@Override
	public long getGroupId(int arg0) {
		return 0;
	}

	@Override
	public View getGroupView(int arg0, boolean arg1, View arg2, ViewGroup arg3) {
		ViewHolder viewHolder = null;

		if (arg2 == null) {
			viewHolder = new ViewHolder();

			arg2 = mExpandableInflater.inflate(
					android.R.layout.simple_list_item_1, null);
			viewHolder.text1 = (TextView) arg2.findViewById(android.R.id.text1);

			arg2.setTag(viewHolder);
		} else {
			viewHolder = (ViewHolder) arg2.getTag();
		}

		viewHolder.text1.setText(mParentStrList.get(arg0));
		return arg2;
	}

	@Override
	public boolean hasStableIds() {
		return false;
	}

	@Override
	public boolean isChildSelectable(int arg0, int arg1) {
		return false;
	}

	private static final int THIS_YEAR = 0; // 本年
	private static final int TODAY = 1; // 当天
	private static final int PREVIOUS_YEAR = 2; // 往年
	private static final int YESTERDAY = 3; // 昨天
	private static final int BEFORE_FEWMINUTES = 4; // 几分钟之前
	private static final int BEFORE_FEWHOURS = 5; // 几小时之前
	private static final int THIS_WEEK = 6; // 本周

	private String[] mStringArray = { "应用版本日期显示", "日历 桌面小工具日期显示", "通话记录列表日期显示",
			"通话记录列表（新）日期显示", "联系人生日日期显示", "联系人生日（新）日期显示", "邮件详情列表日期显示",
			"短信详情列表日期显示", "普通列表日期显示", "个人足迹日期显示", "录音机备忘录列表日期显示",
			"录音机通话录音列表日期显示" };

	private int[] mIntArray = { DateTimeUtils.FORMAT_TYPE_APP_VERSIONS,
			DateTimeUtils.FORMAT_TYPE_CALENDAR_APPWIDGET,
			DateTimeUtils.FORMAT_TYPE_CALL_LOGS,
			DateTimeUtils.FORMAT_TYPE_CALL_LOGS_NEW,
			DateTimeUtils.FORMAT_TYPE_CONTACTS_BIRTHDAY_MD,
			DateTimeUtils.FORMAT_TYPE_CONTACTS_BIRTHDAY_YMD,
			DateTimeUtils.FORMAT_TYPE_EMAIL, DateTimeUtils.FORMAT_TYPE_MMS,
			DateTimeUtils.FORMAT_TYPE_NORMAL,
			DateTimeUtils.FORMAT_TYPE_PERSONAL_FOOTPRINT,
			DateTimeUtils.FORMAT_TYPE_RECORDER,
			DateTimeUtils.FORMAT_TYPE_RECORDER_PHONE };

	private void initParentData() {
		mParentStrList = new ArrayList<String>();
		mChildStrList = new ArrayList<ArrayList<String>>();
		for (int i = 0; i < mStringArray.length; i++) {
			mParentStrList.add(mStringArray[i]);
			mChildStrList.add(initChildData(mIntArray[i]));
		}
	}

	private ArrayList<String> initChildData(int dateType) {
		ArrayList<String> childStrList = new ArrayList<String>();
		switch (dateType) {
		case DateTimeUtils.FORMAT_TYPE_APP_VERSIONS:
		case DateTimeUtils.FORMAT_TYPE_CALENDAR_APPWIDGET:
		case DateTimeUtils.FORMAT_TYPE_CALL_LOGS:
		case DateTimeUtils.FORMAT_TYPE_CALL_LOGS_NEW:
			childStrList.add(dateTimeType(dateType, THIS_YEAR));
			childStrList.add(dateTimeType(dateType, PREVIOUS_YEAR));
			break;
			
		case DateTimeUtils.FORMAT_TYPE_CONTACTS_BIRTHDAY_MD:
		case DateTimeUtils.FORMAT_TYPE_CONTACTS_BIRTHDAY_YMD:
			childStrList.add(dateTimeType(dateType, THIS_YEAR));
			break;
			
		case DateTimeUtils.FORMAT_TYPE_EMAIL:
			childStrList.add(dateTimeType(dateType, THIS_YEAR));
			childStrList.add(dateTimeType(dateType, PREVIOUS_YEAR));
			break;
			
		case DateTimeUtils.FORMAT_TYPE_MMS:
		case DateTimeUtils.FORMAT_TYPE_NORMAL:
			childStrList.add(dateTimeType(dateType, TODAY));
			childStrList.add(dateTimeType(dateType, THIS_WEEK));
			childStrList.add(dateTimeType(dateType, THIS_YEAR));
			childStrList.add(dateTimeType(dateType, PREVIOUS_YEAR));
			break;
			
		case DateTimeUtils.FORMAT_TYPE_PERSONAL_FOOTPRINT:
			childStrList.add(dateTimeType(dateType, BEFORE_FEWMINUTES));
			childStrList.add(dateTimeType(dateType, BEFORE_FEWHOURS));
			childStrList.add(dateTimeType(dateType, YESTERDAY));
			childStrList.add(dateTimeType(dateType, THIS_YEAR));
			childStrList.add(dateTimeType(dateType, PREVIOUS_YEAR));
			break;
			
		case DateTimeUtils.FORMAT_TYPE_RECORDER:
		case DateTimeUtils.FORMAT_TYPE_RECORDER_PHONE:

			childStrList.add(dateTimeType(dateType, TODAY));
			childStrList.add(dateTimeType(dateType, THIS_YEAR));
			childStrList.add(dateTimeType(dateType, PREVIOUS_YEAR));
			break;
		}
		return childStrList;
	}

	private String dateTimeType(int dateType, int timeType) {
		switch (timeType) {
		case THIS_YEAR:
			return "本年："
					+ DateTimeUtils.formatTimeStampString(mExpandableCt,
							System.currentTimeMillis() - 604800000, dateType);
		case TODAY:
			return "当天："
					+ DateTimeUtils.formatTimeStampString(mExpandableCt,
							System.currentTimeMillis(), dateType);
		case PREVIOUS_YEAR:
			return "往年："
					+ DateTimeUtils.formatTimeStampString(mExpandableCt,
							System.currentTimeMillis() / 2, dateType);
		case YESTERDAY:
			return "昨天："
					+ DateTimeUtils.formatTimeStampString(mExpandableCt,
							System.currentTimeMillis() - 86400000, dateType);
		case BEFORE_FEWMINUTES:
			return "xx分钟前："
					+ DateTimeUtils.formatTimeStampString(mExpandableCt,
							System.currentTimeMillis() - 120000, dateType);
		case BEFORE_FEWHOURS:
			return "xx小时前："
					+ DateTimeUtils.formatTimeStampString(mExpandableCt,
							System.currentTimeMillis() - 3610000, dateType);
		case THIS_WEEK:
			return "本周："
					+ DateTimeUtils.formatTimeStampString(mExpandableCt,
							System.currentTimeMillis() - 86400000, dateType);
		}
		return null;
	}

	static class ViewHolder {
		public TextView text1;
	}
}
